@@ -34,6 +34,7 @@
 #include <sys/stat.h>
 #include <sys/fanotify.h>
 #include <sys/time.h>
+#include <ctype.h>
 
 #define BUFSIZE 256*1024
 
@@ -45,6 +46,7 @@
 
 /* command line options */
 static char* option_output = NULL;
+static long option_filter_mask = 0xffffffff;
 static long option_timeout = -1;
 static int option_current_mount = 0;
 static int option_timestamp = 0;
@@ -97,6 +99,9 @@ print_event(const struct fanotify_event_metadata *data,
     static char pathname[PATH_MAX];
     struct stat st;
 
+    if ((data->mask & option_filter_mask) == 0)
+        return;
+
     /* read process name */
     snprintf (printbuf, sizeof (printbuf), "/proc/%i/comm", data->pid);
     len = 0;
@@ -212,6 +217,7 @@ help (void)
 "  -s SECONDS, --seconds=SECONDS\tStop after the given number of seconds.\n"
 "  -t, --timestamp\t\tAdd timestamp to events. Give twice for seconds since the epoch.\n"
 "  -p PID, --ignore-pid PID\tIgnore events for this process ID. Can be specified multiple times.\n"
+"  -f TYPES, --filter=TYPES\tShow only the given event types; choose from C, R, O, or W, e. g. --filter=OC.\n"
 "  -h, --help\t\t\tShow help.");
 }
 
@@ -224,6 +230,7 @@ static void
 parse_args (int argc, char** argv)
 {
     int c;
+    int j;
     long pid;
     char *endptr;
 
@@ -233,12 +240,13 @@ parse_args (int argc, char** argv)
         {"seconds",       required_argument, 0, 's'},
         {"timestamp",     no_argument,       0, 't'},
         {"ignore-pid",    required_argument, 0, 'p'},
+        {"filter",        required_argument, 0, 'f'},
         {"help",          no_argument,       0, 'h'},
         {0,               0,                 0,  0 }
     };
 
     while (1) {
-        c = getopt_long (argc, argv, "co:s:tp:h", long_options, NULL);
+        c = getopt_long (argc, argv, "co:s:tpf:h", long_options, NULL);
 
         if (c == -1)
             break;
@@ -252,6 +260,33 @@ parse_args (int argc, char** argv)
                 option_output = strdup (optarg);
                 break;
 
+            case 'f':
+                j = 0;
+                option_filter_mask = 0;
+                while (optarg[j] != '\0') {
+                    switch (toupper (optarg[j])) {
+                        case 'R':
+                            option_filter_mask |= FAN_ACCESS;
+                            break;
+                        case 'C':
+                            option_filter_mask |= FAN_CLOSE_WRITE;
+                            option_filter_mask |= FAN_CLOSE_NOWRITE;
+                            break;
+                        case 'W':
+                            option_filter_mask |= FAN_CLOSE_WRITE;
+                            option_filter_mask |= FAN_MODIFY;
+                            break;
+                        case 'O':
+                            option_filter_mask |= FAN_OPEN;
+                            break;
+                        default:
+                            fprintf (stderr, "Error: Unknown --filter type '%c'\n", optarg[j]);
+                            exit (1);
+                    }
+                    j++;
+                }
+                break;
+
             case 's':
                 option_timeout = strtol (optarg, &endptr, 10);
                 if (*endptr != '\0' || option_timeout <= 0) {
